# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron import v2_0 as neutronV20
from neutronclient.neutron.v2_0 import subnet
from neutronclient.tests.unit import test_cli20


class CLITestV20SubnetJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['subnet']

    def setUp(self):
        super(CLITestV20SubnetJSON, self).setUp(plurals={'tags': 'tag'})

    def test_create_subnet(self):
        # Create subnet: --gateway gateway netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = '10.10.10.0/24'
        gateway = 'gatewayvalue'
        args = ['--gateway', gateway, netid, cidr, '--description', 'cave']
        position_names = ['ip_version', 'network_id', 'cidr', 'gateway_ip']
        position_values = [4, netid, cidr, gateway]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='cave')

    def test_create_subnet_network_cidr_seperated(self):
        # For positional value, network_id and cidr can be separated.
        # Create subnet: --gateway gateway netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = '10.10.10.0/24'
        gateway = 'gatewayvalue'
        args = [netid, '--gateway', gateway, cidr]
        position_names = ['ip_version', 'network_id', 'cidr', 'gateway_ip']
        position_values = [4, netid, cidr, gateway]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnet_with_no_gateway(self):
        # Create subnet: --no-gateway netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'cidrvalue'
        args = ['--no-gateway', netid, cidr]
        position_names = ['ip_version', 'network_id', 'cidr', 'gateway_ip']
        position_values = [4, netid, cidr, None]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnet_with_segment(self):
        # Create subnet: --segment segment netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = '10.10.10.0/24'
        segment = 'segment'
        args = ['--segment', segment, netid, cidr,
                '--description', 'cave']
        position_names = ['ip_version', 'network_id', 'cidr', 'segment_id']
        position_values = [4, netid, cidr, segment]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='cave')

    def test_create_subnet_with_bad_gateway_option(self):
        # Create sbunet: --no-gateway netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'cidrvalue'
        gateway = 'gatewayvalue'
        args = ['--gateway', gateway, '--no-gateway', netid, cidr]
        position_names = ['ip_version', 'network_id', 'cidr', 'gateway_ip']
        position_values = [4, netid, cidr, None]
        self.assertRaises(
            SystemExit, self._test_create_resource,
            resource, cmd, name, myid, args, position_names, position_values)

    def _test_create_resource_and_catch_command_error(self, should_fail,
                                                      *args):
        if should_fail:
            params = {'no_api_call': True,
                      'expected_exception': exceptions.CommandError}
        else:
            params = {}
        self._test_create_resource(*args, **params)

    def test_create_subnet_with_enable_and_disable_dhcp(self):
        # Create subnet: --enable-dhcp and --disable-dhcp.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'cidrvalue'
        position_names = ['ip_version', 'network_id', 'cidr', 'enable_dhcp']
        # enable_dhcp value is appended later inside the loop
        position_values = [4, netid, cidr]
        for enable_dhcp_arg, should_fail in (
                ('--enable-dhcp=False', False),
                ('--enable-dhcp=True', True),
                ('--enable-dhcp', True)
        ):
            tested_args = [enable_dhcp_arg, '--disable-dhcp']
            args = tested_args + [netid, cidr]
            pos_values = position_values + [should_fail]
            self._test_create_resource_and_catch_command_error(
                should_fail,
                resource, cmd, name, myid, args, position_names, pos_values)

    def test_create_subnet_with_multiple_enable_dhcp(self):
        # Create subnet with multiple --enable-dhcp arguments passed.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'cidrvalue'
        position_names = ['ip_version', 'network_id', 'cidr', 'enable_dhcp']
        # enable_dhcp value is appended later inside the loop
        position_values = [4, netid, cidr]

        _ = 'UNUSED_MARKER'
        for tested_args, should_fail, pos_value in (
                (['--enable-dhcp', '--enable-dhcp=True'], False, True),
                (['--enable-dhcp', '--enable-dhcp=False'], True, _),
                (['--enable-dhcp=False', '--enable-dhcp'], True, _),
                (['--enable-dhcp=True', '--enable-dhcp=False'], True, _),
                (['--enable-dhcp=False', '--enable-dhcp=True'], True, _)
        ):
            args = tested_args + [netid, cidr]
            pos_values = position_values + [pos_value]
            self._test_create_resource_and_catch_command_error(
                should_fail,
                resource, cmd, name, myid, args, position_names, pos_values)

    def test_create_subnet_tenant(self):
        # Create subnet: --tenant_id tenantid netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid', netid, cidr]
        position_names = ['ip_version', 'network_id', 'cidr']
        position_values = [4, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_tags(self):
        # Create subnet: netid cidr --tags a b.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = [netid, cidr, '--tags', 'a', 'b']
        position_names = ['ip_version', 'network_id', 'cidr']
        position_values = [4, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tags=['a', 'b'])

    def test_create_subnet_allocation_pool(self):
        # Create subnet: --tenant_id tenantid <pool> netid cidr.
        # The <pool> is --allocation_pool start=1.1.1.10,end=1.1.1.20
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--allocation_pool', 'start=1.1.1.10,end=1.1.1.20',
                netid, cidr]
        position_names = ['ip_version', 'allocation_pools', 'network_id',
                          'cidr']
        pool = [{'start': '1.1.1.10', 'end': '1.1.1.20'}]
        position_values = [4, pool, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_allocation_pools(self):
        # Create subnet: --tenant-id tenantid <pools> netid cidr.
        # The <pools> are --allocation_pool start=1.1.1.10,end=1.1.1.20 and
        # --allocation_pool start=1.1.1.30,end=1.1.1.40
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--allocation_pool', 'start=1.1.1.10,end=1.1.1.20',
                '--allocation_pool', 'start=1.1.1.30,end=1.1.1.40',
                netid, cidr]
        position_names = ['ip_version', 'allocation_pools', 'network_id',
                          'cidr']
        pools = [{'start': '1.1.1.10', 'end': '1.1.1.20'},
                 {'start': '1.1.1.30', 'end': '1.1.1.40'}]
        position_values = [4, pools, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_host_route(self):
        # Create subnet: --tenant_id tenantid <host_route> netid cidr.
        # The <host_route> is
        # --host-route destination=172.16.1.0/24,nexthop=1.1.1.20
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--host-route', 'destination=172.16.1.0/24,nexthop=1.1.1.20',
                netid, cidr]
        position_names = ['ip_version', 'host_routes', 'network_id',
                          'cidr']
        route = [{'destination': '172.16.1.0/24', 'nexthop': '1.1.1.20'}]
        position_values = [4, route, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_host_routes(self):
        # Create subnet: --tenant-id tenantid <host_routes> netid cidr.
        # The <host_routes> are
        # --host-route destination=172.16.1.0/24,nexthop=1.1.1.20 and
        # --host-route destination=172.17.7.0/24,nexthop=1.1.1.40
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--host-route', 'destination=172.16.1.0/24,nexthop=1.1.1.20',
                '--host-route', 'destination=172.17.7.0/24,nexthop=1.1.1.40',
                netid, cidr]
        position_names = ['ip_version', 'host_routes', 'network_id',
                          'cidr']
        routes = [{'destination': '172.16.1.0/24', 'nexthop': '1.1.1.20'},
                  {'destination': '172.17.7.0/24', 'nexthop': '1.1.1.40'}]
        position_values = [4, routes, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_dns_nameservers(self):
        # Create subnet: --tenant-id tenantid <dns-nameservers> netid cidr.
        # The <dns-nameservers> are
        # --dns-nameserver 1.1.1.20 and --dns-nameserver 1.1.1.40
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--dns-nameserver', '1.1.1.20',
                '--dns-nameserver', '1.1.1.40',
                netid, cidr]
        position_names = ['ip_version', 'dns_nameservers', 'network_id',
                          'cidr']
        nameservers = ['1.1.1.20', '1.1.1.40']
        position_values = [4, nameservers, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_with_use_default_subnetpool(self):
        # Create subnet: --tenant-id tenantid --use-default-subnetpool \
        #     netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--use-default-subnetpool',
                netid, cidr]
        position_names = ['ip_version', 'use_default_subnetpool', 'network_id',
                          'cidr']
        position_values = [4, True, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_with_disable_dhcp(self):
        # Create subnet: --tenant-id tenantid --disable-dhcp netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--disable-dhcp',
                netid, cidr]
        position_names = ['ip_version', 'enable_dhcp', 'network_id',
                          'cidr']
        position_values = [4, False, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_merge_single_plurar(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--allocation-pool', 'start=1.1.1.10,end=1.1.1.20',
                netid, cidr,
                '--allocation-pools', 'list=true', 'type=dict',
                'start=1.1.1.30,end=1.1.1.40']
        position_names = ['ip_version', 'allocation_pools', 'network_id',
                          'cidr']
        pools = [{'start': '1.1.1.10', 'end': '1.1.1.20'},
                 {'start': '1.1.1.30', 'end': '1.1.1.40'}]
        position_values = [4, pools, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_merge_plurar(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                netid, cidr,
                '--allocation-pools', 'list=true', 'type=dict',
                'start=1.1.1.30,end=1.1.1.40']
        position_names = ['ip_version', 'allocation_pools', 'network_id',
                          'cidr']
        pools = [{'start': '1.1.1.30', 'end': '1.1.1.40'}]
        position_values = [4, pools, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_merge_single_single(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--allocation-pool', 'start=1.1.1.10,end=1.1.1.20',
                netid, cidr,
                '--allocation-pool',
                'start=1.1.1.30,end=1.1.1.40']
        position_names = ['ip_version', 'allocation_pools', 'network_id',
                          'cidr']
        pools = [{'start': '1.1.1.10', 'end': '1.1.1.20'},
                 {'start': '1.1.1.30', 'end': '1.1.1.40'}]
        position_values = [4, pools, netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_max_v4_cidr(self):
        # Create subnet: --gateway gateway netid cidr.
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = '192.168.0.1/32'
        gateway = 'gatewayvalue'
        args = ['--gateway', gateway, netid, cidr]
        position_names = ['ip_version', 'network_id', 'cidr', 'gateway_ip']
        position_values = [4, netid, cidr, gateway]
        with mock.patch.object(cmd.log, 'warning') as mock_warning:
            self._test_create_resource(resource, cmd, name, myid, args,
                                       position_names, position_values)
        mock_warning.assert_called_once_with(mock.ANY,
                                             {'ip': 4, 'cidr': '/32'})

    def test_create_subnet_with_ipv6_ra_mode(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '6',
                '--ipv6-ra-mode', 'dhcpv6-stateful',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_ra_mode',
                          'network_id', 'cidr']
        position_values = [6, 'dhcpv6-stateful', netid, cidr]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_with_ipv6_address_mode(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '6',
                '--ipv6-address-mode', 'dhcpv6-stateful',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_address_mode',
                          'network_id', 'cidr']
        position_values = [6, 'dhcpv6-stateful', netid, cidr]

        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_with_ipv6_modes(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '6',
                '--ipv6-address-mode', 'slaac',
                '--ipv6-ra-mode', 'slaac',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_address_mode',
                          'ipv6_ra_mode', 'network_id', 'cidr']
        position_values = [6, 'slaac', 'slaac', netid, cidr]

        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   tenant_id='tenantid')

    def test_create_subnet_with_ipv6_ra_mode_ipv4(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '4',
                '--ipv6-ra-mode', 'slaac',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_ra_mode',
                          'network_id', 'cidr']
        position_values = [4, None, netid, cidr]
        self._test_create_resource(
            resource, cmd, name, myid, args, position_names,
            position_values, tenant_id='tenantid',
            no_api_call=True, expected_exception=exceptions.CommandError)

    def test_create_subnet_with_ipv6_address_mode_ipv4(self):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'prefixvalue'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '4',
                '--ipv6-address-mode', 'slaac',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_address_mode',
                          'network_id', 'cidr']
        position_values = [4, None, netid, cidr]
        self._test_create_resource(
            resource, cmd, name, myid, args, position_names,
            position_values, tenant_id='tenantid',
            no_api_call=True, expected_exception=exceptions.CommandError)

    @mock.patch.object(neutronV20, 'find_resource_by_name_or_id')
    def test_create_subnet_with_subnetpool_ipv6_and_ip_ver_ignored(
            self, mock_find_resource):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '4',
                '--subnetpool', 'subnetpool_id',
                netid]
        position_names = ['ip_version', 'network_id', 'subnetpool_id']
        position_values = [6, netid, 'subnetpool_id']
        mock_find_resource.return_value = {
            'id': 'subnetpool_id', 'ip_version': 6}
        self._test_create_resource(
            resource, cmd, name, myid, args, position_names,
            position_values, tenant_id='tenantid')
        mock_find_resource.assert_called_once_with(
            self.client, 'subnetpool', 'subnetpool_id')

    @mock.patch.object(neutronV20, 'find_resource_by_name_or_id')
    def test_create_subnet_with_subnetpool_ipv4_with_cidr_wildcard(
            self, mock_find_resource):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        cidr = 'cidrwildcard'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '4',
                '--ipv6-address-mode', 'slaac',
                '--subnetpool', 'subnetpool_id',
                netid, cidr]
        position_names = ['ip_version', 'ipv6_address_mode',
                          'network_id', 'subnetpool_id', 'cidr']
        position_values = [4, None, netid, 'subnetpool_id', cidr]
        mock_find_resource.return_value = {'id': 'subnetpool_id',
                                           'ip_version': 4}
        self._test_create_resource(
            resource, cmd, name, myid, args, position_names,
            position_values, tenant_id='tenantid',
            no_api_call=True, expected_exception=exceptions.CommandError)
        mock_find_resource.assert_called_once_with(
            self.client, 'subnetpool', 'subnetpool_id')

    @mock.patch.object(neutronV20, 'find_resource_by_name_or_id')
    def test_create_subnet_with_subnetpool_ipv4_with_prefixlen(
            self, mock_find_resource):
        resource = 'subnet'
        cmd = subnet.CreateSubnet(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        netid = 'netid'
        args = ['--tenant_id', 'tenantid',
                '--ip-version', '4',
                '--ipv6-address-mode', 'slaac',
                '--subnetpool', 'subnetpool_id',
                '--prefixlen', '31',
                netid]
        position_names = ['ip_version', 'ipv6_address_mode',
                          'network_id', 'subnetpool_id']
        position_values = [4, None, netid, 'subnetpool_id']
        mock_find_resource.return_value = {'id': 'subnetpool_id',
                                           'ip_version': 4}
        self._test_create_resource(
            resource, cmd, name, myid, args, position_names,
            position_values, tenant_id='tenantid',
            no_api_call=True, expected_exception=exceptions.CommandError)
        mock_find_resource.assert_called_once_with(
            self.client, 'subnetpool', 'subnetpool_id')

    def test_list_subnets_detail(self):
        # List subnets: -D.
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_subnets_tags(self):
        # List subnets: -- --tags a b.
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, tags=['a', 'b'])

    def test_list_subnets_detail_tags(self):
        # List subnets: -D -- --tags a b.
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, detail=True, tags=['a', 'b'])

    def test_list_subnets_fields(self):
        # List subnets: --fields a --fields b -- --fields c d.
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  fields_1=['a', 'b'], fields_2=['c', 'd'])

    def test_list_subnets_pagination(self):
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_subnets_sort(self):
        # List subnets: --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_subnets_limit(self):
        # List subnets: -P.
        resources = "subnets"
        cmd = subnet.ListSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_update_subnet(self):
        # Update subnet: myid --name myname --tags a b.
        resource = 'subnet'
        cmd = subnet.UpdateSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--description', 'cavern',
                                    '--tags', 'a', 'b'],
                                   {'name': 'myname', 'tags': ['a', 'b'],
                                    'description': 'cavern'})

    def test_update_subnet_allocation_pools(self):
        # Update subnet: myid --name myname --tags a b.
        resource = 'subnet'
        cmd = subnet.UpdateSubnet(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--allocation-pool',
                                    'start=1.2.0.2,end=1.2.0.127'],
                                   {'allocation_pools': [{'start': '1.2.0.2',
                                                          'end': '1.2.0.127'}]}
                                   )

    def test_update_subnet_enable_disable_dhcp(self):
        # Update sbunet: --enable-dhcp and --disable-dhcp.
        resource = 'subnet'
        cmd = subnet.UpdateSubnet(test_cli20.MyApp(sys.stdout), None)
        self.assertRaises(exceptions.CommandError,
                          self._test_update_resource,
                          resource, cmd, 'myid',
                          ['myid', '--name', 'myname', '--enable-dhcp',
                           '--disable-dhcp'], {'name': 'myname', })

    def test_show_subnet(self):
        # Show subnet: --fields id --fields name myid.
        resource = 'subnet'
        cmd = subnet.ShowSubnet(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_delete_subnet(self):
        # Delete subnet: subnetid.
        resource = 'subnet'
        cmd = subnet.DeleteSubnet(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)



# Copyright 2014 Blue Box Group, Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.lb.v2 import member
from neutronclient.tests.unit import test_cli20


class CLITestV20LbMemberJSON(test_cli20.CLITestV20Base):

    def test_create_member_with_mandatory_params(self):
        # lbaas-member-create with mandatory params only.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        cmd = member.CreateMember(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        address = '10.1.1.1'
        protocol_port = '80'
        pool_id = 'pool-id'
        subnet_id = 'subnet-id'
        args = ['--address', address, '--protocol-port', protocol_port,
                '--subnet', subnet_id, pool_id]
        position_names = ['admin_state_up', 'address',
                          'protocol_port', 'subnet_id']
        position_values = [True, address, protocol_port, subnet_id]
        self._test_create_resource(resource, cmd, '', my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)

    def test_create_member_with_all_params(self):
        # lbaas-member-create with all params set.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        cmd = member.CreateMember(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        address = '10.1.1.1'
        protocol_port = '80'
        pool_id = 'pool-id'
        subnet_id = 'subnet-id'
        weight = '100'
        name = 'member1'
        args = ['--address', address, '--protocol-port', protocol_port,
                '--subnet', subnet_id, pool_id, '--weight', weight,
                '--admin-state-down', '--name', name]
        position_names = ['admin_state_up', 'address', 'protocol_port',
                          'subnet_id', 'weight', 'name']
        position_values = [False, address, protocol_port,
                           subnet_id, weight, name]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)

    def test_list_members(self):
        # lbaas-member-list.
        resources = 'members'
        cmd_resources = 'lbaas_members'
        pool_id = 'pool-id'
        cmd = member.ListMember(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True, base_args=[pool_id],
                                  cmd_resources=cmd_resources,
                                  parent_id=pool_id,
                                  query="pool_id=%s" % pool_id)

    def test_list_members_pagination(self):
        # lbaas-member-list with pagination.
        resources = 'members'
        cmd_resources = 'lbaas_members'
        pool_id = 'pool-id'
        cmd = member.ListMember(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd,
                                                  base_args=[pool_id],
                                                  cmd_resources=cmd_resources,
                                                  parent_id=pool_id,
                                                  query="pool_id=%s" % pool_id)

    def test_list_members_sort(self):
        # lbaas-member-list --sort-key id --sort-key asc.
        resources = 'members'
        cmd_resources = 'lbaas_members'
        pool_id = 'pool-id'
        cmd = member.ListMember(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True, base_args=[pool_id],
                                  cmd_resources=cmd_resources,
                                  parent_id=pool_id,
                                  query="pool_id=%s" % pool_id)

    def test_list_members_limit(self):
        # lbaas-member-list -P.
        resources = 'members'
        cmd_resources = 'lbaas_members'
        pool_id = 'pool-id'
        cmd = member.ListMember(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000,
                                  base_args=[pool_id],
                                  cmd_resources=cmd_resources,
                                  parent_id=pool_id,
                                  query="pool_id=%s" % pool_id)

    def test_show_member_id(self):
        # lbaas-member-show test_id.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        pool_id = 'pool-id'
        cmd = member.ShowMember(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id, pool_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'],
                                 cmd_resource=cmd_resource, parent_id=pool_id)

    def test_show_member_id_name(self):
        # lbaas-member-show.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        pool_id = 'pool-id'
        cmd = member.ShowMember(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id, pool_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'],
                                 cmd_resource=cmd_resource, parent_id=pool_id)

    def test_update_member(self):
        # lbaas-member-update myid --name newname.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        my_id = 'my-id'
        pool_id = 'pool-id'
        args = [my_id, pool_id, '--name', 'newname']
        cmd = member.UpdateMember(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, my_id, args,
                                   {'name': 'newname', },
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)
        # lbaas-member-update myid --weight 100.
        args = [my_id, pool_id, '--weight', '100']
        self._test_update_resource(resource, cmd, my_id, args,
                                   {'weight': '100', },
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)
        # lbaas-member-update myid --admin-state-up False
        args = [my_id, pool_id, '--admin-state-up', 'False']
        self._test_update_resource(resource, cmd, my_id, args,
                                   {'admin_state_up': 'False', },
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)

    def test_delete_member(self):
        # lbaas-member-delete my-id.
        resource = 'member'
        cmd_resource = 'lbaas_member'
        cmd = member.DeleteMember(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        pool_id = 'pool-id'
        args = [my_id, pool_id]
        self._test_delete_resource(resource, cmd, my_id, args,
                                   cmd_resource=cmd_resource,
                                   parent_id=pool_id)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron.v2_0 import network
from neutronclient.neutron.v2_0 import tag
from neutronclient import shell
from neutronclient.tests.unit import test_cli20


class CLITestV20Tag(test_cli20.CLITestV20Base):
    def _test_tag_operation(self, cmd, path, method, args, prog_name,
                            body=None):
        with mock.patch.object(cmd, 'get_client',
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, 'request',
                                  return_value=(test_cli20.MyResp(204), None)
                                  ) as mock_request:
            if body:
                body = test_cli20.MyComparator(body, self.client)
            cmd_parser = cmd.get_parser(prog_name)
            shell.run_command(cmd, cmd_parser, args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(test_cli20.end_url(path), self.client),
            method, body=body,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

    def _test_tags_query(self, cmd, resources, args, query):
        path = getattr(self.client, resources + "_path")
        res = {resources: [{'id': 'myid'}]}
        resstr = self.client.serialize(res)
        with mock.patch.object(cmd, 'get_client',
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, 'request',
                                  return_value=(test_cli20.MyResp(200), resstr)
                                  ) as mock_request:
            cmd_parser = cmd.get_parser("list_networks")
            shell.run_command(cmd, cmd_parser, args)

        mock_get_client.assert_called_once_with()
        mock_request.assert_called_once_with(
            test_cli20.MyUrlComparator(test_cli20.end_url(path, query),
                                       self.client),
            'GET', body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))

        _str = self.fake_stdout.make_string()
        self.assertIn('myid', _str)

    def _make_tag_path(self, resource, resource_id, tag):
        path = getattr(self.client, "tag_path")
        resource_plural = self.client.get_resource_plural(resource)
        return path % (resource_plural, resource_id, tag)

    def _make_tags_path(self, resource, resource_id):
        path = getattr(self.client, "tags_path")
        resource_plural = self.client.get_resource_plural(resource)
        return path % (resource_plural, resource_id)

    def test_add_tag(self):
        cmd = tag.AddTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tag_path('network', 'myid', 'red')
        args = ['--resource-type', 'network', '--resource', 'myid',
                '--tag', 'red']
        self._test_tag_operation(cmd, path, 'PUT', args, "tag-add")

    def test_add_tag_empty_tag(self):
        cmd = tag.AddTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tag_path('network', 'myid', '')
        args = ['--resource-type', 'network', '--resource', 'myid',
                '--tag', '']
        self.assertRaises(exceptions.CommandError, self._test_tag_operation,
                          cmd, path, 'PUT', args, "tag-add")

    def test_replace_tag(self):
        cmd = tag.ReplaceTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tags_path('network', 'myid')
        args = ['--resource-type', 'network', '--resource', 'myid',
                '--tag', 'red', '--tag', 'blue']
        body = {'tags': ['red', 'blue']}
        self._test_tag_operation(cmd, path, 'PUT', args, "tag-replace",
                                 body=body)

    def test_remove_tag(self):
        cmd = tag.RemoveTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tag_path('network', 'myid', 'red')
        args = ['--resource-type', 'network', '--resource', 'myid',
                '--tag', 'red']
        self._test_tag_operation(cmd, path, 'DELETE', args, "tag-remove")

    def test_remove_tag_all(self):
        cmd = tag.RemoveTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tags_path('network', 'myid')
        args = ['--resource-type', 'network', '--resource', 'myid',
                '--all']
        self._test_tag_operation(cmd, path, 'DELETE', args, "tag-remove")

    def test_no_tag_nor_all(self):
        cmd = tag.RemoveTag(test_cli20.MyApp(sys.stdout), None)
        path = self._make_tags_path('network', 'myid')
        args = ['--resource-type', 'network', '--resource', 'myid']
        self.assertRaises(exceptions.CommandError, self._test_tag_operation,
                          cmd, path, 'DELETE', args, "tag-remove")

    def test_tags_query(self):
        # This test examines that '-' in the tag related filters
        # is not converted to '_'.
        resources = 'networks'
        cmd = network.ListNetwork(test_cli20.MyApp(sys.stdout), None)
        with mock.patch.object(network.ListNetwork, 'extend_list'):
            args = ['--not-tags', 'red,blue', '--tags-any', 'green',
                    '--not-tags-any', 'black']
            query = "not-tags=red,blue&tags-any=green&not-tags-any=black"
            self._test_tags_query(cmd, resources, args, query)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from neutronclient.neutron.v2_0 import network_ip_availability
from neutronclient.tests.unit import test_cli20


class CLITestV20NetworkIPAvailability(test_cli20.CLITestV20Base):

    id_field = 'network_id'

    def _test_list_network_ip_availability(self, args, query):
        resources = "network_ip_availabilities"
        cmd = network_ip_availability.ListIpAvailability(test_cli20.MyApp
                                                         (sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  base_args=args,
                                  query=query)

    def test_list_network_ip_availability(self):
        self._test_list_network_ip_availability(args=None,
                                                query='ip_version=4')

    def test_list_network_ip_availability_ipv6(self):
        self._test_list_network_ip_availability(
            args=['--ip-version', '6'], query='ip_version=6')

    def test_list_network_ip_availability_net_id_and_ipv4(self):
        self._test_list_network_ip_availability(
            args=['--ip-version', '4', '--network-id', 'myid'],
            query='ip_version=4&network_id=myid')

    def test_list_network_ip_availability_net_name_and_tenant_id(self):
        self._test_list_network_ip_availability(
            args=['--network-name', 'foo', '--tenant-id', 'mytenant'],
            query='network_name=foo&tenant_id=mytenant&ip_version=4')

    def test_show_network_ip_availability(self):
        resource = "network_ip_availability"
        cmd = network_ip_availability.ShowIpAvailability(
            test_cli20.MyApp(sys.stdout), None)
        self._test_show_resource(resource, cmd, self.test_id,
                                 args=[self.test_id])



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

from neutronclient._i18n import _
from neutronclient.neutron import v2_0 as neutronV20


def _format_provider(pool):
    return pool.get('provider') or 'N/A'


class ListPool(neutronV20.ListCommand):
    """List pools that belong to a given tenant."""

    resource = 'pool'
    list_columns = ['id', 'name', 'provider', 'lb_method', 'protocol',
                    'admin_state_up', 'status']
    _formatters = {'provider': _format_provider}
    pagination_support = True
    sorting_support = True


class ShowPool(neutronV20.ShowCommand):
    """Show information of a given pool."""

    resource = 'pool'


class CreatePool(neutronV20.CreateCommand):
    """Create a pool."""

    resource = 'pool'

    def add_known_arguments(self, parser):
        parser.add_argument(
            '--admin-state-down',
            dest='admin_state', action='store_false',
            help=_('Set admin state up to false.'))
        parser.add_argument(
            '--description',
            help=_('Description of the pool.'))
        parser.add_argument(
            '--lb-method',
            required=True,
            choices=['ROUND_ROBIN', 'LEAST_CONNECTIONS', 'SOURCE_IP'],
            help=_('The algorithm used to distribute load between the members '
                   'of the pool.'))
        parser.add_argument(
            '--name',
            required=True,
            help=_('The name of the pool.'))
        parser.add_argument(
            '--protocol',
            required=True,
            choices=['HTTP', 'HTTPS', 'TCP'],
            help=_('Protocol for balancing.'))
        parser.add_argument(
            '--subnet-id', metavar='SUBNET',
            required=True,
            help=_('The subnet on which the members of the pool will be '
                   'located.'))
        parser.add_argument(
            '--provider',
            help=_('Provider name of the loadbalancer service.'))

    def args2body(self, parsed_args):
        _subnet_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'subnet', parsed_args.subnet_id)
        body = {'admin_state_up': parsed_args.admin_state,
                'subnet_id': _subnet_id}
        neutronV20.update_dict(parsed_args, body,
                               ['description', 'lb_method', 'name',
                                'protocol', 'tenant_id', 'provider'])
        return {self.resource: body}


class UpdatePool(neutronV20.UpdateCommand):
    """Update a given pool."""

    resource = 'pool'


class DeletePool(neutronV20.DeleteCommand):
    """Delete a given pool."""

    resource = 'pool'


class RetrievePoolStats(neutronV20.ShowCommand):
    """Retrieve stats for a given pool."""

    resource = 'pool'

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        pool_id = neutronV20.find_resourceid_by_name_or_id(
            self.get_client(), 'pool', parsed_args.id)
        params = {}
        if parsed_args.fields:
            params = {'fields': parsed_args.fields}

        data = neutron_client.retrieve_pool_stats(pool_id, **params)
        self.format_output_data(data)
        stats = data['stats']
        if 'stats' in data:
            return zip(*sorted(stats.items()))
        else:
            return None



# Copyright 2016-2017 FUJITSU LIMITED
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

FWG = 'firewall_group'
FWGS = 'firewall_groups'
FWP = 'firewall_policy'
FWPS = 'firewall_policies'
FWR = 'firewall_rule'
FWRS = 'firewall_rules'
CMD_FWG = 'fwaas_' + FWG
CMD_FWP = 'fwaas_' + FWP
CMD_FWR = 'fwaas_' + FWR



# Copyright 2011 VMware, Inc
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import encodeutils

from neutronclient._i18n import _

"""
Neutron base exception handling.

Exceptions are classified into three categories:
* Exceptions corresponding to exceptions from neutron server:
  This type of exceptions should inherit one of exceptions
  in HTTP_EXCEPTION_MAP.
* Exceptions from client library:
  This type of exceptions should inherit NeutronClientException.
* Exceptions from CLI code:
  This type of exceptions should inherit NeutronCLIError.
"""


# NOTE: This method is defined here to avoid
# an import loop between common.utils and this module.
def _safe_decode_dict(kwargs):
    for k, v in kwargs.items():
        kwargs[k] = encodeutils.safe_decode(v)
    return kwargs


class NeutronException(Exception):
    """Base Neutron Exception.

    To correctly use this class, inherit from it and define
    a 'message' property. That message will get printf'd
    with the keyword arguments provided to the constructor.
    """
    message = _("An unknown exception occurred.")

    def __init__(self, message=None, **kwargs):
        if message:
            self.message = message
        try:
            self._error_string = self.message % _safe_decode_dict(kwargs)
        except Exception:
            # at least get the core message out if something happened
            self._error_string = self.message

    def __str__(self):
        return self._error_string


class NeutronClientException(NeutronException):
    """Base exception which exceptions from Neutron are mapped into.

    NOTE: on the client side, we use different exception types in order
    to allow client library users to handle server exceptions in try...except
    blocks. The actual error message is the one generated on the server side.
    """

    status_code = 0
    req_ids_msg = _("Neutron server returns request_ids: %s")
    request_ids = []

    def __init__(self, message=None, **kwargs):
        self.request_ids = kwargs.get('request_ids')
        if 'status_code' in kwargs:
            self.status_code = kwargs['status_code']
        if self.request_ids:
            req_ids_msg = self.req_ids_msg % self.request_ids
            if message:
                message = _('%(msg)s\n%(id)s') % {'msg': message,
                                                  'id': req_ids_msg}
            else:
                message = req_ids_msg
        super(NeutronClientException, self).__init__(message, **kwargs)


# Base exceptions from Neutron

class BadRequest(NeutronClientException):
    status_code = 400


class Unauthorized(NeutronClientException):
    status_code = 401
    message = _("Unauthorized: bad credentials.")


class Forbidden(NeutronClientException):
    status_code = 403
    message = _("Forbidden: your credentials don't give you access to this "
                "resource.")


class NotFound(NeutronClientException):
    status_code = 404


class Conflict(NeutronClientException):
    status_code = 409


class InternalServerError(NeutronClientException):
    status_code = 500


class ServiceUnavailable(NeutronClientException):
    status_code = 503


HTTP_EXCEPTION_MAP = {
    400: BadRequest,
    401: Unauthorized,
    403: Forbidden,
    404: NotFound,
    409: Conflict,
    500: InternalServerError,
    503: ServiceUnavailable,
}


# Exceptions mapped to Neutron server exceptions
# These are defined if a user of client library needs specific exception.
# Exception name should be <Neutron Exception Name> + 'Client'
# e.g., NetworkNotFound -> NetworkNotFoundClient

class NetworkNotFoundClient(NotFound):
    pass


class PortNotFoundClient(NotFound):
    pass


class StateInvalidClient(BadRequest):
    pass


class NetworkInUseClient(Conflict):
    pass


class PortInUseClient(Conflict):
    pass


class IpAddressInUseClient(Conflict):
    pass


class IpAddressAlreadyAllocatedClient(Conflict):
    pass


class InvalidIpForNetworkClient(BadRequest):
    pass


class InvalidIpForSubnetClient(BadRequest):
    pass


class OverQuotaClient(Conflict):
    pass


class IpAddressGenerationFailureClient(Conflict):
    pass


class MacAddressInUseClient(Conflict):
    pass


class HostNotCompatibleWithFixedIpsClient(Conflict):
    pass


class ExternalIpAddressExhaustedClient(BadRequest):
    pass


# Exceptions from client library

class NoAuthURLProvided(Unauthorized):
    message = _("auth_url was not provided to the Neutron client")


class EndpointNotFound(NeutronClientException):
    message = _("Could not find Service or Region in Service Catalog.")


class EndpointTypeNotFound(NeutronClientException):
    message = _("Could not find endpoint type %(type_)s in Service Catalog.")


class AmbiguousEndpoints(NeutronClientException):
    message = _("Found more than one matching endpoint in Service Catalog: "
                "%(matching_endpoints)")


class RequestURITooLong(NeutronClientException):
    """Raised when a request fails with HTTP error 414."""

    def __init__(self, **kwargs):
        self.excess = kwargs.get('excess', 0)
        super(RequestURITooLong, self).__init__(**kwargs)


class ConnectionFailed(NeutronClientException):
    message = _("Connection to neutron failed: %(reason)s")


class SslCertificateValidationError(NeutronClientException):
    message = _("SSL certificate validation has failed: %(reason)s")


class MalformedResponseBody(NeutronClientException):
    message = _("Malformed response body: %(reason)s")


class InvalidContentType(NeutronClientException):
    message = _("Invalid content type %(content_type)s.")


# Command line exceptions

class NeutronCLIError(NeutronException):
    """Exception raised when command line parsing fails."""
    pass


class CommandError(NeutronCLIError):
    pass


class UnsupportedVersion(NeutronCLIError):
    """Indicates usage of an unsupported API version

    Indicates that the user is trying to use an unsupported version of
    the API.
    """
    pass


class NeutronClientNoUniqueMatch(NeutronCLIError):
    message = _("Multiple %(resource)s matches found for name '%(name)s',"
                " use an ID to be more specific.")



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.vpn import vpnservice
from neutronclient.tests.unit import test_cli20


class CLITestV20VpnServiceJSON(test_cli20.CLITestV20Base):

    def test_create_vpnservice_all_params(self):
        # vpn-service-create all params.
        resource = 'vpnservice'
        cmd = vpnservice.CreateVPNService(test_cli20.MyApp(sys.stdout), None)
        subnet = 'mysubnet-id'
        router = 'myrouter-id'
        tenant_id = 'mytenant-id'
        my_id = 'my-id'
        name = 'myvpnservice'
        description = 'my-vpn-service'
        admin_state = True

        args = ['--name', name,
                '--description', description,
                router,
                subnet,
                '--tenant-id', tenant_id]

        position_names = ['admin_state_up', 'name', 'description',
                          'subnet_id', 'router_id',
                          'tenant_id']

        position_values = [admin_state, name, description,
                           subnet, router, tenant_id]

        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_vpnservice_with_limited_params(self):
        # vpn-service-create with limited params.
        resource = 'vpnservice'
        cmd = vpnservice.CreateVPNService(test_cli20.MyApp(sys.stdout), None)
        subnet = 'mysubnet-id'
        router = 'myrouter-id'
        tenant_id = 'mytenant-id'
        my_id = 'my-id'
        admin_state = True

        args = [router,
                subnet,
                '--tenant-id', tenant_id]

        position_names = ['admin_state_up',
                          'subnet_id', 'router_id',
                          'tenant_id']

        position_values = [admin_state, subnet, router, tenant_id]

        self._test_create_resource(resource, cmd, None, my_id, args,
                                   position_names, position_values)

    def test_create_vpnservice_without_subnet(self):
        # vpn-service-create with no subnet provided.
        resource = 'vpnservice'
        cmd = vpnservice.CreateVPNService(test_cli20.MyApp(sys.stdout), None)
        router = 'myrouter-id'
        tenant_id = 'mytenant-id'
        my_id = 'my-id'
        admin_state = True

        args = [router,
                '--tenant-id', tenant_id]

        position_names = ['admin_state_up',
                          'subnet_id', 'router_id',
                          'tenant_id']

        position_values = [admin_state, None, router, tenant_id]

        self._test_create_resource(resource, cmd, None, my_id, args,
                                   position_names, position_values)

    def test_list_vpnservice(self):
        # vpn-service-list.
        resources = "vpnservices"
        cmd = vpnservice.ListVPNService(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_vpnservice_pagination(self):
        # vpn-service-list.
        resources = "vpnservices"
        cmd = vpnservice.ListVPNService(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_vpnservice_sort(self):
        # vpn-service-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "vpnservices"
        cmd = vpnservice.ListVPNService(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_vpnservice_limit(self):
        # vpn-service-list -P.
        resources = "vpnservices"
        cmd = vpnservice.ListVPNService(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_vpnservice_id(self):
        # vpn-service-show test_id.
        resource = 'vpnservice'
        cmd = vpnservice.ShowVPNService(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_vpnservice_id_name(self):
        # vpn-service-show."""
        resource = 'vpnservice'
        cmd = vpnservice.ShowVPNService(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_vpnservice(self):
        # vpn-service-update myid --name newname --tags a b.
        resource = 'vpnservice'
        cmd = vpnservice.UpdateVPNService(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })
        # vpn-service-update myid --admin-state-up False
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'False'],
                                   {'admin_state_up': 'False', })

    def test_delete_vpnservice(self):
        # vpn-service-delete my-id.
        resource = 'vpnservice'
        cmd = vpnservice.DeleteVPNService(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    (c) Copyright 2013 Hewlett-Packard Development Company, L.P.
#    All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.common import exceptions
from neutronclient.neutron.v2_0.vpn import ipsecpolicy
from neutronclient.tests.unit import test_cli20


class CLITestV20VpnIpsecPolicyJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['ipsecpolicy']

    def _test_create_ipsecpolicy_all_params(self, auth='sha1',
                                            expected_exc=None):
        # vpn-ipsecpolicy-create all params with dashes.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.CreateIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        name = 'ipsecpolicy1'
        description = 'first-ipsecpolicy1'
        auth_algorithm = auth
        encryption_algorithm = 'aes-256'
        encapsulation_mode = 'tunnel'
        pfs = 'group5'
        transform_protocol = 'ah'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        lifetime = 'units=seconds,value=20000'

        args = [name,
                '--description', description,
                '--tenant-id', tenant_id,
                '--auth-algorithm', auth_algorithm,
                '--encryption-algorithm', encryption_algorithm,
                '--transform-protocol', transform_protocol,
                '--encapsulation-mode', encapsulation_mode,
                '--lifetime', lifetime,
                '--pfs', pfs]

        position_names = ['name', 'auth_algorithm', 'encryption_algorithm',
                          'encapsulation_mode', 'description',
                          'transform_protocol', 'pfs',
                          'tenant_id']

        position_values = [name, auth_algorithm, encryption_algorithm,
                           encapsulation_mode, description,
                           transform_protocol, pfs,
                           tenant_id]
        extra_body = {
            'lifetime': {
                'units': 'seconds',
                'value': 20000,
            },
        }

        if not expected_exc:
            self._test_create_resource(resource, cmd, name, my_id, args,
                                       position_names, position_values,
                                       extra_body=extra_body)
        else:
            self.assertRaises(
                expected_exc,
                self._test_create_resource,
                resource, cmd, name, my_id, args,
                position_names, position_values,
                extra_body=extra_body)

    def test_create_ipsecpolicy_all_params(self):
        self._test_create_ipsecpolicy_all_params()

    def test_create_ipsecpolicy_auth_sha256(self):
        self._test_create_ipsecpolicy_all_params(auth='sha256')

    def test_create_ipsecpolicy_auth_sha384(self):
        self._test_create_ipsecpolicy_all_params(auth='sha384')

    def test_create_ipsecpolicy_auth_sha512(self):
        self._test_create_ipsecpolicy_all_params(auth='sha512')

    def test_create_ipsecpolicy_invalid_auth(self):
        self._test_create_ipsecpolicy_all_params(auth='invalid',
                                                 expected_exc=SystemExit)

    def test_create_ipsecpolicy_with_limited_params(self):
        # vpn-ipsecpolicy-create with limited params.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.CreateIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        name = 'ipsecpolicy1'
        auth_algorithm = 'sha1'
        encryption_algorithm = 'aes-128'
        encapsulation_mode = 'tunnel'
        pfs = 'group5'
        transform_protocol = 'esp'
        tenant_id = 'my-tenant'
        my_id = 'my-id'

        args = [name,
                '--tenant-id', tenant_id]

        position_names = ['name', 'auth_algorithm', 'encryption_algorithm',
                          'encapsulation_mode',
                          'transform_protocol', 'pfs',
                          'tenant_id']

        position_values = [name, auth_algorithm, encryption_algorithm,
                           encapsulation_mode,
                           transform_protocol, pfs,
                           tenant_id]

        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def _test_lifetime_values(self, lifetime, expected_exc=None):
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.CreateIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        name = 'ipsecpolicy1'
        description = 'my-ipsec-policy'
        auth_algorithm = 'sha1'
        encryption_algorithm = 'aes-256'
        ike_version = 'v1'
        phase1_negotiation_mode = 'main'
        pfs = 'group5'
        tenant_id = 'my-tenant'
        my_id = 'my-id'

        args = [name,
                '--description', description,
                '--tenant-id', tenant_id,
                '--auth-algorithm', auth_algorithm,
                '--encryption-algorithm', encryption_algorithm,
                '--ike-version', ike_version,
                '--phase1-negotiation-mode', phase1_negotiation_mode,
                '--lifetime', lifetime,
                '--pfs', pfs]

        position_names = ['name', 'description',
                          'auth_algorithm', 'encryption_algorithm',
                          'phase1_negotiation_mode',
                          'ike_version', 'pfs',
                          'tenant_id']

        position_values = [name, description,
                           auth_algorithm, encryption_algorithm,
                           phase1_negotiation_mode, ike_version, pfs,
                           tenant_id]
        if not expected_exc:
            expected_exc = exceptions.CommandError
        self.assertRaises(
            expected_exc,
            self._test_create_resource,
            resource, cmd, name, my_id, args,
            position_names, position_values)

    def test_create_ipsecpolicy_with_invalid_lifetime_keys(self):
        lifetime = 'uts=seconds,val=20000'
        self._test_lifetime_values(lifetime, SystemExit)

    def test_create_ipsecpolicy_with_invalid_lifetime_units(self):
        lifetime = 'units=minutes,value=600'
        self._test_lifetime_values(lifetime)

    def test_create_ipsecpolicy_with_invalid_lifetime_value(self):
        lifetime = 'units=seconds,value=0'
        self._test_lifetime_values(lifetime)

    def test_list_ipsecpolicy(self):
        # vpn-ipsecpolicy-list.
        resources = "ipsecpolicies"
        cmd = ipsecpolicy.ListIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_ipsecpolicy_pagination(self):
        # vpn-ipsecpolicy-list.
        resources = "ipsecpolicies"
        cmd = ipsecpolicy.ListIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_ipsecpolicy_sort(self):
        # vpn-ipsecpolicy-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "ipsecpolicies"
        cmd = ipsecpolicy.ListIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_ipsecpolicy_limit(self):
        # vpn-ipsecpolicy-list -P.
        resources = "ipsecpolicies"
        cmd = ipsecpolicy.ListIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_ipsecpolicy_id(self):
        # vpn-ipsecpolicy-show ipsecpolicy_id.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.ShowIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_ipsecpolicy_id_name(self):
        # vpn-ipsecpolicy-show.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.ShowIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_ipsecpolicy_name(self):
        # vpn-ipsecpolicy-update myid --name newname --tags a b.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.UpdateIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })

    def test_update_ipsecpolicy_other_params(self):
        # vpn-ipsecpolicy-update myid --transform-protocol esp
        # --pfs group14  --encapsulation-mode transport
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.UpdateIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--transform-protocol', 'esp',
                                    '--pfs', 'group14',
                                    '--encapsulation-mode', 'transport'],
                                   {'transform_protocol': 'esp',
                                    'pfs': 'group14',
                                    'encapsulation_mode': 'transport', })

    def test_delete_ipsecpolicy(self):
        # vpn-ipsecpolicy-delete my-id.
        resource = 'ipsecpolicy'
        cmd = ipsecpolicy.DeleteIPsecPolicy(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#
from unittest import mock

from neutronclient.osc.v2.dynamic_routing import bgp_peer
from neutronclient.tests.unit.osc.v2.dynamic_routing import fakes


class TestListBgpPeer(fakes.TestNeutronDynamicRoutingOSCV2):
    _bgp_peers = fakes.FakeBgpPeer.create_bgp_peers(count=1)
    columns = ('ID', 'Name', 'Peer IP', 'Remote AS')
    data = []
    for _bgp_peer in _bgp_peers['bgp_peers']:
        data.append((
            _bgp_peer['id'],
            _bgp_peer['name'],
            _bgp_peer['peer_ip'],
            _bgp_peer['remote_as']))

    def setUp(self):
        super(TestListBgpPeer, self).setUp()

        self.neutronclient.list_bgp_peers = mock.Mock(
            return_value=self._bgp_peers
        )

        # Get the command object to test
        self.cmd = bgp_peer.ListBgpPeer(self.app, self.namespace)

    def test_bgp_peer_list(self):
        parsed_args = self.check_parser(self.cmd, [], [])

        columns, data = self.cmd.take_action(parsed_args)
        self.neutronclient.list_bgp_peers.assert_called_once_with()
        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestDeleteBgpPeer(fakes.TestNeutronDynamicRoutingOSCV2):

    _bgp_peer = fakes.FakeBgpPeer.create_one_bgp_peer()

    def setUp(self):
        super(TestDeleteBgpPeer, self).setUp()

        self.neutronclient.delete_bgp_peer = mock.Mock(return_value=None)

        self.cmd = bgp_peer.DeleteBgpPeer(self.app, self.namespace)

    def test_delete_bgp_peer(self):
        arglist = [
            self._bgp_peer['name'],
        ]
        verifylist = [
            ('bgp_peer', self._bgp_peer['name']),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.neutronclient.delete_bgp_peer.assert_called_once_with(
            self._bgp_peer['name'])
        self.assertIsNone(result)


class TestShowBgpPeer(fakes.TestNeutronDynamicRoutingOSCV2):
    _one_bgp_peer = fakes.FakeBgpPeer.create_one_bgp_peer()
    data = (
        _one_bgp_peer['auth_type'],
        _one_bgp_peer['id'],
        _one_bgp_peer['name'],
        _one_bgp_peer['peer_ip'],
        _one_bgp_peer['remote_as'],
        _one_bgp_peer['tenant_id']
    )
    _bgp_peer = {'bgp_peer': _one_bgp_peer}
    _bgp_peer_name = _one_bgp_peer['name']
    columns = (
        'auth_type',
        'id',
        'name',
        'peer_ip',
        'remote_as',
        'tenant_id'
    )

    def setUp(self):
        super(TestShowBgpPeer, self).setUp()

        self.neutronclient.show_bgp_peer = mock.Mock(
            return_value=self._bgp_peer
        )
        bgp_peer.get_bgp_peer_id = mock.Mock(return_value=self._bgp_peer_name)
        # Get the command object to test
        self.cmd = bgp_peer.ShowBgpPeer(self.app, self.namespace)

    def test_bgp_peer_list(self):
        arglist = [
            self._bgp_peer_name,
        ]
        verifylist = [
            ('bgp_peer', self._bgp_peer_name),
        ]
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        data = self.cmd.take_action(parsed_args)
        self.neutronclient.show_bgp_peer.assert_called_once_with(
            self._bgp_peer_name)
        self.assertEqual(self.columns, data[0])
        self.assertEqual(self.data, data[1])


class TestSetBgpPeer(fakes.TestNeutronDynamicRoutingOSCV2):
    _one_bgp_peer = fakes.FakeBgpPeer.create_one_bgp_peer()
    _bgp_peer_name = _one_bgp_peer['name']

    def setUp(self):
        super(TestSetBgpPeer, self).setUp()
        self.neutronclient.update_bgp_peer = mock.Mock(return_value=None)
        bgp_peer.get_bgp_peer_id = mock.Mock(return_value=self._bgp_peer_name)

        self.cmd = bgp_peer.SetBgpPeer(self.app, self.namespace)

    def test_set_bgp_peer(self):
        arglist = [
            self._bgp_peer_name,
            '--name', 'noob',
        ]
        verifylist = [
            ('bgp_peer', self._bgp_peer_name),
            ('name', 'noob'),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)
        result = self.cmd.take_action(parsed_args)

        attrs = {'bgp_peer': {
            'name': 'noob',
            'password': None}
        }
        self.neutronclient.update_bgp_peer.assert_called_once_with(
            self._bgp_peer_name, attrs)
        self.assertIsNone(result)



# Copyright 2016 IBM
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import argparse

from cliff import show
from oslo_serialization import jsonutils

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.neutron import v2_0


class ShowAutoAllocatedTopology(v2_0.NeutronCommand, show.ShowOne):
    """Show the auto-allocated topology of a given tenant."""

    resource = 'auto_allocated_topology'

    def get_parser(self, prog_name):
        parser = super(ShowAutoAllocatedTopology, self).get_parser(prog_name)
        parser.add_argument(
            '--dry-run',
            help=_('Validate the requirements for auto-allocated-topology. '
                   '(Does not return a topology.)'),
            action='store_true')
        parser.add_argument(
            '--tenant-id', metavar='tenant-id',
            help=_('The owner tenant ID.'))
        # Allow people to do
        #    neutron auto-allocated-topology-show <tenant-id>
        # (Only useful to users who can look at other tenants' topologies.)
        # We use a different name for this arg because the default will
        # override whatever is in the named arg otherwise.
        parser.add_argument(
            'pos_tenant_id',
            help=argparse.SUPPRESS, nargs='?')
        return parser

    def take_action(self, parsed_args):
        client = self.get_client()
        extra_values = v2_0.parse_args_to_dict(self.values_specs)
        if extra_values:
            raise exceptions.CommandError(
                _("Invalid argument(s): --%s") % ', --'.join(extra_values))
        tenant_id = parsed_args.tenant_id or parsed_args.pos_tenant_id
        if parsed_args.dry_run:
            data = client.validate_auto_allocated_topology_requirements(
                tenant_id)
        else:
            data = client.get_auto_allocated_topology(tenant_id)
        if self.resource in data:
            for k, v in data[self.resource].items():
                if isinstance(v, list):
                    value = ""
                    for _item in v:
                        if value:
                            value += "\n"
                        if isinstance(_item, dict):
                            value += jsonutils.dumps(_item)
                        else:
                            value += str(_item)
                    data[self.resource][k] = value
                elif v == "dry-run=pass":
                    return ("dry-run",), ("pass",)
                elif v is None:
                    data[self.resource][k] = ''
            return zip(*sorted(data[self.resource].items()))
        else:
            return None


class DeleteAutoAllocatedTopology(v2_0.NeutronCommand):
    """Delete the auto-allocated topology of a given tenant."""

    resource = 'auto_allocated_topology'

    def get_parser(self, prog_name):
        parser = super(DeleteAutoAllocatedTopology, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant-id', metavar='tenant-id',
            help=_('The owner tenant ID.'))
        # Allow people to do
        #    neutron auto-allocated-topology-delete <tenant-id>
        # (Only useful to users who can look at other tenants' topologies.)
        # We use a different name for this arg because the default will
        # override whatever is in the named arg otherwise.
        parser.add_argument(
            'pos_tenant_id',
            help=argparse.SUPPRESS, nargs='?')
        return parser

    def take_action(self, parsed_args):
        client = self.get_client()
        tenant_id = parsed_args.tenant_id or parsed_args.pos_tenant_id
        client.delete_auto_allocated_topology(tenant_id)
        # It tenant is None, let's be clear on what it means.
        tenant_id = tenant_id or 'None (i.e. yours)'
        print(_('Deleted topology for tenant %s.') % tenant_id,
              file=self.app.stdout)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import copy
from unittest import mock

from oslo_utils import uuidutils


class FakeTrunk(object):
    """Fake one or more trunks."""
    @staticmethod
    def create_one_trunk(attrs=None):
        """Create a fake trunk.

        :param Dictionary attrs:
            A dictionary with all attributes
        :return:
            A Dictionary with id, name, description, admin_state_up, port_id,
            sub_ports, status and project_id
        """
        attrs = attrs or {}

        # Set default attributes.
        trunk_attrs = {
            'id': 'trunk-id-' + uuidutils.generate_uuid(dashed=False),
            'name': 'trunk-name-' + uuidutils.generate_uuid(dashed=False),
            'description': '',
            'port_id': 'port-' + uuidutils.generate_uuid(dashed=False),
            'admin_state_up': True,
            'project_id': 'project-id-' +
            uuidutils.generate_uuid(dashed=False),
            'status': 'ACTIVE',
            'sub_ports': [{'port_id': 'subport-' +
                           uuidutils.generate_uuid(dashed=False),
                           'segmentation_type': 'vlan',
                           'segmentation_id': 100}],
        }

        # Overwrite default attributes.
        trunk_attrs.update(attrs)
        return copy.deepcopy(trunk_attrs)

    @staticmethod
    def create_trunks(attrs=None, count=2):
        """Create multiple fake trunks.

        :param Dictionary attrs:
            A dictionary with all attributes
        :param int count:
            The number of routers to fake
        :return:
            A list of dictionaries faking the trunks
        """
        trunks = []
        for i in range(0, count):
            trunks.append(FakeTrunk.create_one_trunk(attrs))

        return trunks

    @staticmethod
    def get_trunks(trunks=None, count=2):
        """Get an iterable Mock object with a list of faked trunks.

        If trunks list is provided, then initialize the Mock object with the
        list. Otherwise create one.

        :param List trunks:
            A list of FakeResource objects faking trunks
        :param int count:
            The number of trunks to fake
        :return:
            An iterable Mock object with side_effect set to a list of faked
            trunks
        """
        if trunks is None:
            trunks = FakeTrunk.create_trunks(count)
        return mock.Mock(side_effect=trunks)



# Copyright 2016 Cisco Systems
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0 import purge
from neutronclient.tests.unit import test_cli20


class CLITestV20Purge(test_cli20.CLITestV20Base):

    def setUp(self):
        super(CLITestV20Purge, self).setUp()
        self.resource_types = ['floatingip', 'port', 'router',
                               'network', 'security_group']

    def _generate_resources_dict(self, value=0):
        resources_dict = {}
        resources_dict['true'] = value
        for resource_type in self.resource_types:
            resources_dict[resource_type] = value
        return resources_dict

    def _verify_suffix(self, resources, message):
        for resource, value in resources.items():
            if value > 0:
                suffix = list('%(value)d %(resource)s' %
                              {'value': value, 'resource': resource})
                if value != 1:
                    suffix.append('s')
                suffix = ''.join(suffix)
                self.assertIn(suffix, message)
            else:
                self.assertNotIn(resource, message)

    def _verify_message(self, message, deleted, failed):
        message = message.split('.')
        success_prefix = "Deleted "
        failure_prefix = "The following resources could not be deleted: "
        if not deleted['true']:
            for msg in message:
                self.assertNotIn(success_prefix, msg)
            message = message[0]
            if not failed['true']:
                expected = 'Tenant has no supported resources'
                self.assertEqual(expected, message)
            else:
                self.assertIn(failure_prefix, message)
                self._verify_suffix(failed, message)
        else:
            resources_deleted = message[0]
            self.assertIn(success_prefix, resources_deleted)
            self._verify_suffix(deleted, resources_deleted)
            if failed['true']:
                resources_failed = message[1]
                self.assertIn(failure_prefix, resources_failed)
                self._verify_suffix(failed, resources_failed)
            else:
                for msg in message:
                    self.assertNotIn(failure_prefix, msg)

    def _verify_result(self, my_purge, deleted, failed):
        message = my_purge._build_message(deleted, failed, failed['true'])
        self._verify_message(message, deleted, failed)

    def test_build_message(self):
        my_purge = purge.Purge(test_cli20.MyApp(sys.stdout), None)

        # Verify message when tenant has no supported resources
        deleted = self._generate_resources_dict()
        failed = self._generate_resources_dict()
        self._verify_result(my_purge, deleted, failed)

        # Verify message when tenant has supported resources,
        # and they are all deleteable
        deleted = self._generate_resources_dict(1)
        self._verify_result(my_purge, deleted, failed)

        # Verify message when tenant has supported resources,
        # and some are not deleteable
        failed = self._generate_resources_dict(1)
        self._verify_result(my_purge, deleted, failed)

        # Verify message when tenant has supported resources,
        # and all are not deleteable
        deleted = self._generate_resources_dict()
        self._verify_result(my_purge, deleted, failed)



# Copyright (c) 2016 Juniper Networks Inc.
# All Rights Reserved.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#

import copy
import operator
from unittest import mock

from osc_lib import exceptions
from osc_lib import utils as osc_utils
from osc_lib.utils import columns as column_util

from neutronclient.osc.v2.networking_bgpvpn import bgpvpn
from neutronclient.osc.v2.networking_bgpvpn import constants
from neutronclient.tests.unit.osc.v2.networking_bgpvpn import fakes


columns_short = tuple(col for col, _, listing_mode in bgpvpn._attr_map
                      if listing_mode in (column_util.LIST_BOTH,
                                          column_util.LIST_SHORT_ONLY))
columns_long = tuple(col for col, _, listing_mode in bgpvpn._attr_map
                     if listing_mode in (column_util.LIST_BOTH,
                                         column_util.LIST_LONG_ONLY))
headers_short = tuple(head for _, head, listing_mode in bgpvpn._attr_map
                      if listing_mode in (column_util.LIST_BOTH,
                                          column_util.LIST_SHORT_ONLY))
headers_long = tuple(head for _, head, listing_mode in bgpvpn._attr_map
                     if listing_mode in (column_util.LIST_BOTH,
                                         column_util.LIST_LONG_ONLY))
sorted_attr_map = sorted(bgpvpn._attr_map, key=operator.itemgetter(1))
sorted_columns = tuple(col for col, _, _ in sorted_attr_map)
sorted_headers = tuple(head for _, head, _ in sorted_attr_map)


def _get_data(attrs, columns=sorted_columns):
    return osc_utils.get_dict_properties(attrs, columns,
                                         formatters=bgpvpn._formatters)


class TestCreateBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestCreateBgpvpn, self).setUp()
        self.cmd = bgpvpn.CreateBgpvpn(self.app, self.namespace)

    def test_create_bgpvpn_with_no_args(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.neutronclient.create_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        arglist = []
        verifylist = [
            ('project', None),
            ('name', None),
            ('type', 'l3'),
            ('vni', None),
            ('local_pref', None),
            ('route_targets', None),
            ('import_targets', None),
            ('export_targets', None),
            ('route_distinguishers', None),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cols, data = self.cmd.take_action(parsed_args)

        self.neutronclient.create_bgpvpn.assert_called_once_with(
            {constants.BGPVPN: {'type': 'l3'}})
        self.assertEqual(sorted_headers, cols)
        self.assertItemEqual(_get_data(fake_bgpvpn), data)

    def test_create_bgpvpn_with_all_args(self):
        attrs = {
            'tenant_id': 'new_fake_project_id',
            'name': 'fake_name',
            'type': 'l2',
            'vni': 100,
            'local_pref': 777,
            'route_targets': ['fake_rt1', 'fake_rt2', 'fake_rt3'],
            'import_targets': ['fake_irt1', 'fake_irt2', 'fake_irt3'],
            'export_targets': ['fake_ert1', 'fake_ert2', 'fake_ert3'],
            'route_distinguishers': ['fake_rd1', 'fake_rd2', 'fake_rd3'],
        }
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn(attrs)
        self.neutronclient.create_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        arglist = [
            '--project', fake_bgpvpn['tenant_id'],
            '--name', fake_bgpvpn['name'],
            '--type', fake_bgpvpn['type'],
            '--vni', str(fake_bgpvpn['vni']),
            '--local-pref', str(fake_bgpvpn['local_pref']),
        ]
        for rt in fake_bgpvpn['route_targets']:
            arglist.extend(['--route-target', rt])
        for rt in fake_bgpvpn['import_targets']:
            arglist.extend(['--import-target', rt])
        for rt in fake_bgpvpn['export_targets']:
            arglist.extend(['--export-target', rt])
        for rd in fake_bgpvpn['route_distinguishers']:
            arglist.extend(['--route-distinguisher', rd])
        verifylist = [
            ('project', fake_bgpvpn['tenant_id']),
            ('name', fake_bgpvpn['name']),
            ('type', fake_bgpvpn['type']),
            ('vni', fake_bgpvpn['vni']),
            ('local_pref', fake_bgpvpn['local_pref']),
            ('route_targets', fake_bgpvpn['route_targets']),
            ('import_targets', fake_bgpvpn['import_targets']),
            ('export_targets', fake_bgpvpn['export_targets']),
            ('route_distinguishers', fake_bgpvpn['route_distinguishers']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        cols, data = self.cmd.take_action(parsed_args)

        fake_bgpvpn_call = copy.deepcopy(fake_bgpvpn)
        fake_bgpvpn_call.pop('id')
        fake_bgpvpn_call.pop('networks')
        fake_bgpvpn_call.pop('routers')
        fake_bgpvpn_call.pop('ports')

        self.neutronclient.create_bgpvpn.assert_called_once_with(
            {constants.BGPVPN: fake_bgpvpn_call})
        self.assertEqual(sorted_headers, cols)
        self.assertItemEqual(_get_data(fake_bgpvpn), data)


class TestSetBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestSetBgpvpn, self).setUp()
        self.cmd = bgpvpn.SetBgpvpn(self.app, self.namespace)

    def test_set_bgpvpn(self):
        attrs = {
            'route_targets': ['set_rt1', 'set_rt2', 'set_rt3'],
            'import_targets': ['set_irt1', 'set_irt2', 'set_irt3'],
            'export_targets': ['set_ert1', 'set_ert2', 'set_ert3'],
            'route_distinguishers': ['set_rd1', 'set_rd2', 'set_rd3'],
        }
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn(attrs)
        self.neutronclient.show_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        self.neutronclient.update_bgpvpn = mock.Mock()
        arglist = [
            fake_bgpvpn['id'],
            '--name', 'set_name',
            '--route-target', 'set_rt1',
            '--import-target', 'set_irt1',
            '--export-target', 'set_ert1',
            '--route-distinguisher', 'set_rd1',
        ]
        verifylist = [
            ('bgpvpn', fake_bgpvpn['id']),
            ('name', 'set_name'),
            ('route_targets', ['set_rt1']),
            ('purge_route_target', False),
            ('import_targets', ['set_irt1']),
            ('purge_import_target', False),
            ('export_targets', ['set_ert1']),
            ('purge_export_target', False),
            ('route_distinguishers', ['set_rd1']),
            ('purge_route_distinguisher', False),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'name': 'set_name',
            'route_targets': list(set(fake_bgpvpn['route_targets']) |
                                  set(['set_rt1'])),
            'import_targets': list(set(fake_bgpvpn['import_targets']) |
                                   set(['set_irt1'])),
            'export_targets': list(set(fake_bgpvpn['export_targets']) |
                                   set(['set_ert1'])),
            'route_distinguishers': list(
                set(fake_bgpvpn['route_distinguishers']) | set(['set_rd1'])),
        }
        self.neutronclient.update_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'], {constants.BGPVPN: attrs})
        self.assertIsNone(result)

    def test_set_bgpvpn_with_purge_list(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.neutronclient.show_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        self.neutronclient.update_bgpvpn = mock.Mock()
        arglist = [
            fake_bgpvpn['id'],
            '--route-target', 'set_rt1',
            '--no-route-target',
            '--import-target', 'set_irt1',
            '--no-import-target',
            '--export-target', 'set_ert1',
            '--no-export-target',
            '--route-distinguisher', 'set_rd1',
            '--no-route-distinguisher',
        ]
        verifylist = [
            ('bgpvpn', fake_bgpvpn['id']),
            ('route_targets', ['set_rt1']),
            ('purge_route_target', True),
            ('import_targets', ['set_irt1']),
            ('purge_import_target', True),
            ('export_targets', ['set_ert1']),
            ('purge_export_target', True),
            ('route_distinguishers', ['set_rd1']),
            ('purge_route_distinguisher', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'route_targets': [],
            'import_targets': [],
            'export_targets': [],
            'route_distinguishers': [],
        }
        self.neutronclient.update_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'], {constants.BGPVPN: attrs})
        self.assertIsNone(result)


class TestUnsetBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestUnsetBgpvpn, self).setUp()
        self.cmd = bgpvpn.UnsetBgpvpn(self.app, self.namespace)

    def test_unset_bgpvpn(self):
        attrs = {
            'route_targets': ['unset_rt1', 'unset_rt2', 'unset_rt3'],
            'import_targets': ['unset_irt1', 'unset_irt2', 'unset_irt3'],
            'export_targets': ['unset_ert1', 'unset_ert2', 'unset_ert3'],
            'route_distinguishers': ['unset_rd1', 'unset_rd2', 'unset_rd3'],
        }
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn(attrs)
        self.neutronclient.show_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        self.neutronclient.update_bgpvpn = mock.Mock()
        arglist = [
            fake_bgpvpn['id'],
            '--route-target', 'unset_rt1',
            '--import-target', 'unset_irt1',
            '--export-target', 'unset_ert1',
            '--route-distinguisher', 'unset_rd1',
        ]
        verifylist = [
            ('bgpvpn', fake_bgpvpn['id']),
            ('route_targets', ['unset_rt1']),
            ('purge_route_target', False),
            ('import_targets', ['unset_irt1']),
            ('purge_import_target', False),
            ('export_targets', ['unset_ert1']),
            ('purge_export_target', False),
            ('route_distinguishers', ['unset_rd1']),
            ('purge_route_distinguisher', False),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'route_targets': list(set(fake_bgpvpn['route_targets']) -
                                  set(['unset_rt1'])),
            'import_targets': list(set(fake_bgpvpn['import_targets']) -
                                   set(['unset_irt1'])),
            'export_targets': list(set(fake_bgpvpn['export_targets']) -
                                   set(['unset_ert1'])),
            'route_distinguishers': list(
                set(fake_bgpvpn['route_distinguishers']) - set(['unset_rd1'])),
        }
        self.neutronclient.update_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'], {constants.BGPVPN: attrs})
        self.assertIsNone(result)

    def test_unset_bgpvpn_with_purge_list(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.neutronclient.show_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        self.neutronclient.update_bgpvpn = mock.Mock()
        arglist = [
            fake_bgpvpn['id'],
            '--route-target', 'unset_rt1',
            '--all-route-target',
            '--import-target', 'unset_irt1',
            '--all-import-target',
            '--export-target', 'unset_ert1',
            '--all-export-target',
            '--route-distinguisher', 'unset_rd1',
            '--all-route-distinguisher',
        ]
        verifylist = [
            ('bgpvpn', fake_bgpvpn['id']),
            ('route_targets', ['unset_rt1']),
            ('purge_route_target', True),
            ('import_targets', ['unset_irt1']),
            ('purge_import_target', True),
            ('export_targets', ['unset_ert1']),
            ('purge_export_target', True),
            ('route_distinguishers', ['unset_rd1']),
            ('purge_route_distinguisher', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        attrs = {
            'route_targets': [],
            'import_targets': [],
            'export_targets': [],
            'route_distinguishers': [],
        }
        self.neutronclient.update_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'], {constants.BGPVPN: attrs})
        self.assertIsNone(result)


class TestDeleteBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestDeleteBgpvpn, self).setUp()
        self.neutronclient.find_resource = mock.Mock(
            side_effect=lambda _, name_or_id: {'id': name_or_id})
        self.cmd = bgpvpn.DeleteBgpvpn(self.app, self.namespace)

    def test_delete_one_bgpvpn(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.neutronclient.delete_bgpvpn = mock.Mock()
        arglist = [
            fake_bgpvpn['id'],
        ]
        verifylist = [
            ('bgpvpns', [fake_bgpvpn['id']]),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.neutronclient.delete_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'])
        self.assertIsNone(result)

    def test_delete_multi_bpgvpn(self):
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=3)
        fake_bgpvpn_ids = [fake_bgpvpn['id'] for fake_bgpvpn in
                           fake_bgpvpns[constants.BGPVPNS]]
        self.neutronclient.delete_bgpvpn = mock.Mock()
        arglist = fake_bgpvpn_ids
        verifylist = [
            ('bgpvpns', fake_bgpvpn_ids),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        result = self.cmd.take_action(parsed_args)

        self.neutronclient.delete_bgpvpn.assert_has_calls(
            [mock.call(id) for id in fake_bgpvpn_ids])
        self.assertIsNone(result)

    def test_delete_multi_bpgvpn_with_unknown(self):
        count = 3
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=count)
        fake_bgpvpn_ids = [fake_bgpvpn['id'] for fake_bgpvpn in
                           fake_bgpvpns[constants.BGPVPNS]]

        def raise_unknonw_resource(resource_path, name_or_id):
            if str(count - 2) in name_or_id:
                raise Exception()
        self.neutronclient.delete_bgpvpn = mock.Mock(
            side_effect=raise_unknonw_resource)
        arglist = fake_bgpvpn_ids
        verifylist = [
            ('bgpvpns', fake_bgpvpn_ids),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        self.assertRaises(exceptions.CommandError, self.cmd.take_action,
                          parsed_args)

        self.neutronclient.delete_bgpvpn.assert_has_calls(
            [mock.call(id) for id in fake_bgpvpn_ids])


class TestListBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestListBgpvpn, self).setUp()
        self.cmd = bgpvpn.ListBgpvpn(self.app, self.namespace)

    def test_list_all_bgpvpn(self):
        count = 3
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=count)
        self.neutronclient.list_bgpvpns = mock.Mock(return_value=fake_bgpvpns)
        arglist = []
        verifylist = []

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_bgpvpns.assert_called_once()
        self.assertEqual(headers, list(headers_short))
        self.assertListItemEqual(
            list(data),
            [_get_data(fake_bgpvpn, columns_short) for fake_bgpvpn
             in fake_bgpvpns[constants.BGPVPNS]])

    def test_list_all_bgpvpn_long_mode(self):
        count = 3
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=count)
        self.neutronclient.list_bgpvpns = mock.Mock(return_value=fake_bgpvpns)
        arglist = [
            '--long',
        ]
        verifylist = [
            ('long', True),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_bgpvpns.assert_called_once()
        self.assertEqual(headers, list(headers_long))
        self.assertListItemEqual(
            list(data),
            [_get_data(fake_bgpvpn, columns_long) for fake_bgpvpn
             in fake_bgpvpns[constants.BGPVPNS]])

    def test_list_project_bgpvpn(self):
        count = 3
        project_id = 'list_fake_project_id'
        attrs = {'tenant_id': project_id}
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=count,
                                                       attrs=attrs)
        self.neutronclient.list_bgpvpns = mock.Mock(return_value=fake_bgpvpns)
        arglist = [
            '--project', project_id,
        ]
        verifylist = [
            ('project', project_id),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_bgpvpns.assert_called_once_with(
            tenant_id=project_id)
        self.assertEqual(headers, list(headers_short))
        self.assertListItemEqual(
            list(data),
            [_get_data(fake_bgpvpn, columns_short) for fake_bgpvpn
             in fake_bgpvpns[constants.BGPVPNS]])

    def test_list_bgpvpn_with_filters(self):
        count = 3
        name = 'fake_id0'
        layer_type = 'l2'
        attrs = {'type': layer_type}
        fake_bgpvpns = fakes.FakeBgpvpn.create_bgpvpns(count=count,
                                                       attrs=attrs)
        returned_bgpvpn = fake_bgpvpns[constants.BGPVPNS][0]
        self.neutronclient.list_bgpvpns = mock.Mock(
            return_value={constants.BGPVPNS: [returned_bgpvpn]})
        arglist = [
            '--property', 'name=%s' % name,
            '--property', 'type=%s' % layer_type,
        ]
        verifylist = [
            ('property', {'name': name, 'type': layer_type}),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.list_bgpvpns.assert_called_once_with(
            name=name,
            type=layer_type)
        self.assertEqual(headers, list(headers_short))
        self.assertListItemEqual(list(data),
                                 [_get_data(returned_bgpvpn, columns_short)])


class TestShowBgpvpn(fakes.TestNeutronClientBgpvpn):
    def setUp(self):
        super(TestShowBgpvpn, self).setUp()
        self.cmd = bgpvpn.ShowBgpvpn(self.app, self.namespace)

    def test_show_bgpvpn(self):
        fake_bgpvpn = fakes.FakeBgpvpn.create_one_bgpvpn()
        self.neutronclient.show_bgpvpn = mock.Mock(
            return_value={constants.BGPVPN: fake_bgpvpn})
        arglist = [
            fake_bgpvpn['id'],
        ]
        verifylist = [
            ('bgpvpn', fake_bgpvpn['id']),
        ]

        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        headers, data = self.cmd.take_action(parsed_args)

        self.neutronclient.show_bgpvpn.assert_called_once_with(
            fake_bgpvpn['id'])
        self.assertEqual(sorted_headers, headers)
        self.assertItemEqual(_get_data(fake_bgpvpn), data)



# Copyright 2013 Big Switch Networks Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0.fw import firewall
from neutronclient.tests.unit import test_cli20


class CLITestV20FirewallJSON(test_cli20.CLITestV20Base):

    def test_create_firewall_with_mandatory_params(self):
        # firewall-create with mandatory (none) params.
        resource = 'firewall'
        cmd = firewall.CreateFirewall(test_cli20.MyApp(sys.stdout), None)
        name = ''
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        policy_id = 'my-policy-id'
        args = ['--tenant-id', tenant_id, policy_id, ]
        position_names = ['firewall_policy_id', ]
        position_values = [policy_id, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   admin_state_up=True, tenant_id=tenant_id)

    def test_create_firewall_with_all_params(self):
        # firewall-create with all params set.
        resource = 'firewall'
        cmd = firewall.CreateFirewall(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'my-desc'
        policy_id = 'my-policy-id'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--description', description,
                '--admin-state-down',
                '--tenant-id', tenant_id,
                policy_id]
        position_names = ['firewall_policy_id', ]
        position_values = [policy_id, ]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values,
                                   description=description,
                                   admin_state_up=False,
                                   tenant_id=tenant_id)

    def test_create_firewall_with_routers(self):
        resource = 'firewall'
        cmd = firewall.CreateFirewall(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        policy_id = 'my-policy-id'
        my_id = 'my-id'
        args = ['--router', 'fake-id', '--router', 'fake-name', policy_id]
        router_ids = ['fake-id', 'fake-name']
        position_names = ['firewall_policy_id', 'router_ids']
        position_values = [policy_id, router_ids]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_list_firewalls(self):
        # firewall-list.
        resources = "firewalls"
        cmd = firewall.ListFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_firewalls_pagination(self):
        # firewall-list with pagination.
        resources = "firewalls"
        cmd = firewall.ListFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_firewalls_sort(self):
        # sorted list: firewall-list --sort-key name --sort-key id
        # --sort-key asc --sort-key desc
        resources = "firewalls"
        cmd = firewall.ListFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_firewalls_limit(self):
        # size (1000) limited list: firewall-list -P.
        resources = "firewalls"
        cmd = firewall.ListFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_firewall_id(self):
        # firewall-show test_id.
        resource = 'firewall'
        cmd = firewall.ShowFirewall(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_firewall_id_name(self):
        # firewall-show.
        resource = 'firewall'
        cmd = firewall.ShowFirewall(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_firewall(self):
        # firewall-update myid --name newname --tags a b.
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })

    def test_update_firewall_using_policy_name(self):
        # firewall-update myid --policy newpolicy.
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--policy', 'newpolicy'],
                                   {'firewall_policy_id': 'newpolicy'})

    def test_update_firewall_with_routers(self):
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(
            resource, cmd, 'myid',
            ['myid', '--router', 'fake-id', '--router', 'fake-name'],
            {'router_ids': ['fake-id', 'fake-name']})

    def test_update_firewall_with_no_routers(self):
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(
            resource, cmd, 'myid',
            ['myid', '--no-routers'], {'router_ids': []})

    def test_update_firewall_with_bad_router_options(self):
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self.assertRaises(
            SystemExit,
            self._test_update_resource,
            resource, cmd, 'myid',
            ['myid', '--no-routers', '--router', 'fake-id'], {})

    def test_delete_firewall(self):
        # firewall-delete my-id.
        resource = 'firewall'
        cmd = firewall.DeleteFirewall(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_update_firewall_admin_state(self):
        # firewall-update myid --admin-state-up True.
        resource = 'firewall'
        cmd = firewall.UpdateFirewall(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--admin-state-up', 'True'],
                                   {'admin_state_up': 'True'})



# Copyright 2016 IBM
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys

from neutronclient.neutron.v2_0 import auto_allocated_topology as aat
from neutronclient.tests.unit import test_cli20


class TestAutoAllocatedTopologyJSON(test_cli20.CLITestV20Base):

    def test_show_auto_allocated_topology_arg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.ShowAutoAllocatedTopology(test_cli20.MyApp(sys.stdout), None)
        args = ['--tenant-id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args)

    def test_show_auto_allocated_topology_posarg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.ShowAutoAllocatedTopology(test_cli20.MyApp(sys.stdout), None)
        args = ['some-tenant']
        self._test_show_resource(resource, cmd, "some-tenant", args)

    def test_show_auto_allocated_topology_no_arg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.ShowAutoAllocatedTopology(test_cli20.MyApp(sys.stdout), None)
        args = []
        self._test_show_resource(resource, cmd, "None", args)

    def test_show_auto_allocated_topology_dry_run_as_tenant(self):
        resource = 'auto_allocated_topology'
        cmd = aat.ShowAutoAllocatedTopology(test_cli20.MyApp(sys.stdout), None)
        args = ['--dry-run']
        self._test_show_resource(resource, cmd, "None", args,
                                 fields=('dry-run',))

    def test_show_auto_allocated_topology_dry_run_as_admin(self):
        resource = 'auto_allocated_topology'
        cmd = aat.ShowAutoAllocatedTopology(test_cli20.MyApp(sys.stdout), None)
        args = ['--dry-run', 'some-tenant']
        self._test_show_resource(resource, cmd, "some-tenant", args,
                                 fields=('dry-run',))

    def test_delete_auto_allocated_topology_arg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.DeleteAutoAllocatedTopology(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['--tenant-id', self.test_id]
        self._test_delete_resource(resource, cmd, self.test_id, args)

    def test_delete_auto_allocated_topology_posarg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.DeleteAutoAllocatedTopology(test_cli20.MyApp(sys.stdout),
                                              None)
        args = ['some-tenant']
        self._test_delete_resource(resource, cmd, "some-tenant", args)

    def test_delete_auto_allocated_topology_no_arg(self):
        resource = 'auto_allocated_topology'
        cmd = aat.DeleteAutoAllocatedTopology(test_cli20.MyApp(sys.stdout),
                                              None)
        args = []
        self._test_delete_resource(resource, cmd, "None", args)



# Copyright 2015 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.common import exceptions
from neutronclient.neutron.v2_0 import subnetpool
from neutronclient.tests.unit import test_cli20


class CLITestV20SubnetPoolJSON(test_cli20.CLITestV20Base):

    non_admin_status_resources = ['subnetpool']

    def setUp(self):
        super(CLITestV20SubnetPoolJSON, self).setUp(plurals={'tags': 'tag'})

    def test_create_subnetpool_with_options(self):
        # Create subnetpool: myname.
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        min_prefixlen = 30
        prefix1 = '10.11.12.0/24'
        prefix2 = '12.11.13.0/24'
        args = [name, '--min-prefixlen', str(min_prefixlen),
                '--pool-prefix', prefix1, '--pool-prefix', prefix2,
                '--shared', '--description', 'public pool',
                '--tenant-id', 'tenantid']
        position_names = ['name', 'min_prefixlen', 'prefixes', 'shared',
                          'tenant_id']
        position_values = [name, min_prefixlen, [prefix1, prefix2], True,
                           'tenantid']
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values,
                                   description='public pool')

    def test_create_subnetpool_only_with_required_options(self):
        # Create subnetpool: myname.
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        min_prefixlen = 30
        prefix1 = '10.11.12.0/24'
        prefix2 = '12.11.13.0/24'
        args = [name, '--min-prefixlen', str(min_prefixlen),
                '--pool-prefix', prefix1, '--pool-prefix', prefix2]
        position_names = ['name', 'min_prefixlen', 'prefixes']
        position_values = [name, min_prefixlen, [prefix1, prefix2]]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnetpool_with_is_default(self, default='false'):
        # Create subnetpool: myname.
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        min_prefixlen = 30
        prefix1 = '10.11.12.0/24'
        prefix2 = '12.11.13.0/24'
        args = [name, '--min-prefixlen', str(min_prefixlen),
                '--pool-prefix', prefix1, '--pool-prefix', prefix2,
                '--is-default', default]
        position_names = ['name', 'min_prefixlen', 'prefixes', 'is_default']
        position_values = [name, min_prefixlen, [prefix1, prefix2], default]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnetpool_default(self):
        self.test_create_subnetpool_with_is_default(default='true')

    def test_create_subnetpool_with_unicode(self):
        # Create subnetpool: u'\u7f51\u7edc'.
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = u'\u7f51\u7edc'
        myid = 'myid'
        min_prefixlen = 30
        prefixes = '10.11.12.0/24'
        args = [name, '--min-prefixlen', str(min_prefixlen),
                '--pool-prefix', prefixes]
        position_names = ['name', 'min_prefixlen', 'prefixes']
        position_values = [name, min_prefixlen, [prefixes]]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnetpool_with_addrscope(self):
        # Create subnetpool: myname in addrscope: foo-address-scope
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        min_prefixlen = 30
        prefix1 = '11.11.11.0/24'
        prefix2 = '12.12.12.0/24'
        address_scope = 'foo-address-scope'
        args = [name, '--min-prefixlen', str(min_prefixlen),
                '--pool-prefix', prefix1, '--pool-prefix', prefix2,
                '--address-scope', address_scope]
        position_names = ['name', 'min_prefixlen', 'prefixes',
                          'address_scope_id']
        position_values = [name, min_prefixlen, [prefix1, prefix2],
                           address_scope]
        self._test_create_resource(resource, cmd, name, myid, args,
                                   position_names, position_values)

    def test_create_subnetpool_no_poolprefix(self):
        # Should raise an error because --pool-prefix is required
        resource = 'subnetpool'
        cmd = subnetpool.CreateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        name = 'myname'
        myid = 'myid'
        args = [name]
        position_names = ['name']
        position_values = [name]
        self.assertRaises(SystemExit, self._test_create_resource, resource,
                          cmd, name, myid, args, position_names,
                          position_values)

    @mock.patch.object(subnetpool.ListSubnetPool, "extend_list")
    def test_list_subnetpool_pagination(self, mock_extend_list):
        cmd = subnetpool.ListSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination("subnetpools", cmd)
        mock_extend_list.assert_called_once_with(test_cli20.IsA(list),
                                                 mock.ANY)

    def test_list_subnetpools_sort(self):
        # List subnetpools:
        # --sort-key name --sort-key id --sort-key asc --sort-key desc
        resources = "subnetpools"
        cmd = subnetpool.ListSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_subnetpools_limit(self):
        # List subnetpools: -P.
        resources = "subnetpools"
        cmd = subnetpool.ListSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_update_subnetpool_exception(self):
        # Update subnetpool: myid.
        resource = 'subnetpool'
        cmd = subnetpool.UpdateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self.assertRaises(exceptions.CommandError, self._test_update_resource,
                          resource, cmd, 'myid', ['myid'], {})

    def test_update_subnetpool(self):
        # Update subnetpool: myid --name myname.
        resource = 'subnetpool'
        cmd = subnetpool.UpdateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'myname',
                                    '--description', ':)'],
                                   {'name': 'myname', 'description': ':)'})

    def test_update_subnetpool_with_address_scope(self):
        # Update subnetpool: myid --address-scope newscope.
        resource = 'subnetpool'
        cmd = subnetpool.UpdateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--address-scope', 'newscope'],
                                   {'address_scope_id': 'newscope'}
                                   )

    def test_update_subnetpool_with_no_address_scope(self):
        # Update subnetpool: myid --no-address-scope.
        resource = 'subnetpool'
        cmd = subnetpool.UpdateSubnetPool(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--no-address-scope'],
                                   {'address_scope_id': None}
                                   )

    def test_show_subnetpool(self):
        # Show subnetpool: --fields id --fields name myid.
        resource = 'subnetpool'
        cmd = subnetpool.ShowSubnetPool(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args,
                                 ['id', 'name'])

    def test_delete_subnetpool(self):
        # Delete subnetpool: subnetpoolid.
        resource = 'subnetpool'
        cmd = subnetpool.DeleteSubnetPool(test_cli20.MyApp(sys.stdout), None)
        myid = 'myid'
        args = [myid]
        self._test_delete_resource(resource, cmd, myid, args)



# Copyright 2013 Mirantis Inc.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import sys
from unittest import mock


from neutronclient.neutron.v2_0.lb import pool
from neutronclient.tests.unit import test_cli20


class CLITestV20LbPoolJSON(test_cli20.CLITestV20Base):

    def test_create_pool_with_mandatory_params(self):
        # lb-pool-create with mandatory params only.
        resource = 'pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        lb_method = 'ROUND_ROBIN'
        protocol = 'HTTP'
        subnet_id = 'subnet-id'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        args = ['--lb-method', lb_method,
                '--name', name,
                '--protocol', protocol,
                '--subnet-id', subnet_id,
                '--tenant-id', tenant_id]
        position_names = ['admin_state_up', 'lb_method', 'name',
                          'protocol', 'subnet_id', 'tenant_id']
        position_values = [True, lb_method, name,
                           protocol, subnet_id, tenant_id]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_create_pool_with_all_params(self):
        # lb-pool-create with all params set.
        resource = 'pool'
        cmd = pool.CreatePool(test_cli20.MyApp(sys.stdout), None)
        name = 'my-name'
        description = 'my-desc'
        lb_method = 'ROUND_ROBIN'
        protocol = 'HTTP'
        subnet_id = 'subnet-id'
        tenant_id = 'my-tenant'
        my_id = 'my-id'
        provider = 'lbaas'
        args = ['--admin-state-down',
                '--description', description,
                '--lb-method', lb_method,
                '--name', name,
                '--protocol', protocol,
                '--subnet-id', subnet_id,
                '--tenant-id', tenant_id,
                '--provider', provider]
        position_names = ['admin_state_up', 'description', 'lb_method', 'name',
                          'protocol', 'subnet_id', 'tenant_id', 'provider']
        position_values = [False, description, lb_method, name,
                           protocol, subnet_id, tenant_id, provider]
        self._test_create_resource(resource, cmd, name, my_id, args,
                                   position_names, position_values)

    def test_list_pools(self):
        # lb-pool-list.
        resources = "pools"
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, True)

    def test_list_pools_pagination(self):
        # lb-pool-list.
        resources = "pools"
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources_with_pagination(resources, cmd)

    def test_list_pools_sort(self):
        # lb-pool-list --sort-key name --sort-key id --sort-key asc
        # --sort-key desc
        resources = "pools"
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd,
                                  sort_key=["name", "id"],
                                  sort_dir=["asc", "desc"])

    def test_list_pools_limit(self):
        # lb-pool-list -P.
        resources = "pools"
        cmd = pool.ListPool(test_cli20.MyApp(sys.stdout), None)
        self._test_list_resources(resources, cmd, page_size=1000)

    def test_show_pool_id(self):
        # lb-pool-show test_id.
        resource = 'pool'
        cmd = pool.ShowPool(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id, args, ['id'])

    def test_show_pool_id_name(self):
        # lb-pool-show.
        resource = 'pool'
        cmd = pool.ShowPool(test_cli20.MyApp(sys.stdout), None)
        args = ['--fields', 'id', '--fields', 'name', self.test_id]
        self._test_show_resource(resource, cmd, self.test_id,
                                 args, ['id', 'name'])

    def test_update_pool(self):
        # lb-pool-update myid --name newname --tags a b.
        resource = 'pool'
        cmd = pool.UpdatePool(test_cli20.MyApp(sys.stdout), None)
        self._test_update_resource(resource, cmd, 'myid',
                                   ['myid', '--name', 'newname'],
                                   {'name': 'newname', })

    def test_delete_pool(self):
        # lb-pool-delete my-id.
        resource = 'pool'
        cmd = pool.DeletePool(test_cli20.MyApp(sys.stdout), None)
        my_id = 'my-id'
        args = [my_id]
        self._test_delete_resource(resource, cmd, my_id, args)

    def test_retrieve_pool_stats(self):
        # lb-pool-stats test_id.
        resource = 'pool'
        cmd = pool.RetrievePoolStats(test_cli20.MyApp(sys.stdout), None)
        my_id = self.test_id
        fields = ['bytes_in', 'bytes_out']
        args = ['--fields', 'bytes_in', '--fields', 'bytes_out', my_id]

        query = "&".join(["fields=%s" % field for field in fields])
        expected_res = {'stats': {'bytes_in': '1234', 'bytes_out': '4321'}}
        resstr = self.client.serialize(expected_res)
        path = getattr(self.client, "pool_path_stats")
        return_tup = (test_cli20.MyResp(200), resstr)

        cmd_parser = cmd.get_parser("test_" + resource)
        parsed_args = cmd_parser.parse_args(args)

        with mock.patch.object(cmd, "get_client",
                               return_value=self.client) as mock_get_client, \
                mock.patch.object(self.client.httpclient, "request",
                                  return_value=return_tup) as mock_request:
            cmd.run(parsed_args)

        self.assert_mock_multiple_calls_with_same_arguments(
            mock_get_client, mock.call(), 2)
        mock_request.assert_called_once_with(
            test_cli20.end_url(path % my_id, query), 'GET',
            body=None,
            headers=test_cli20.ContainsKeyValue(
                {'X-Auth-Token': test_cli20.TOKEN}))
        _str = self.fake_stdout.make_string()
        self.assertIn('bytes_in', _str)
        self.assertIn('bytes_out', _str)



# Copyright 2012 OpenStack Foundation.
# All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
#

import abc
import argparse

from cliff import lister
from cliff import show
from oslo_serialization import jsonutils

from neutronclient._i18n import _
from neutronclient.common import exceptions
from neutronclient.common import utils
from neutronclient.neutron import v2_0 as neutronV20


def get_tenant_id(args, client):
    return (args.pos_tenant_id or args.tenant_id or
            client.get_quotas_tenant()['tenant']['tenant_id'])


class DeleteQuota(neutronV20.NeutronCommand):
    """Delete defined quotas of a given tenant."""

    resource = 'quota'

    def get_parser(self, prog_name):
        parser = super(DeleteQuota, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant-id', metavar='tenant-id',
            help=_('The owner tenant ID.'))
        parser.add_argument(
            '--tenant_id',
            help=argparse.SUPPRESS)
        parser.add_argument(
            'pos_tenant_id',
            help=argparse.SUPPRESS, nargs='?')
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        tenant_id = get_tenant_id(parsed_args, neutron_client)
        obj_deleter = getattr(neutron_client,
                              "delete_%s" % self.resource)
        obj_deleter(tenant_id)
        print((_('Deleted %(resource)s: %(tenant_id)s')
               % {'tenant_id': tenant_id,
                  'resource': self.resource}),
              file=self.app.stdout)
        return


class ListQuota(neutronV20.NeutronCommand, lister.Lister):
    """List quotas of all tenants who have non-default quota values."""

    resource = 'quota'

    def get_parser(self, prog_name):
        parser = super(ListQuota, self).get_parser(prog_name)
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        search_opts = {}
        self.log.debug('search options: %s', search_opts)
        obj_lister = getattr(neutron_client,
                             "list_%ss" % self.resource)
        data = obj_lister(**search_opts)
        info = []
        collection = self.resource + "s"
        if collection in data:
            info = data[collection]
        _columns = len(info) > 0 and sorted(info[0].keys()) or []
        return (_columns, (utils.get_item_properties(s, _columns)
                           for s in info))


class ShowQuotaBase(neutronV20.NeutronCommand, show.ShowOne):
    """Base class to show quotas of a given tenant."""

    resource = "quota"

    @abc.abstractmethod
    def retrieve_data(self, tenant_id, neutron_client):
        """Retrieve data using neutron client for the given tenant."""

    def get_parser(self, prog_name):
        parser = super(ShowQuotaBase, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant-id', metavar='tenant-id',
            help=_('The owner tenant ID.'))
        parser.add_argument(
            '--tenant_id',
            help=argparse.SUPPRESS)
        # allow people to do neutron quota-show <tenant-id>.
        # we use a different name for this because the default will
        # override whatever is in the named arg otherwise.
        parser.add_argument(
            'pos_tenant_id',
            help=argparse.SUPPRESS, nargs='?')
        return parser

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        tenant_id = get_tenant_id(parsed_args, neutron_client)
        data = self.retrieve_data(tenant_id, neutron_client)
        if self.resource in data:
            return zip(*sorted(data[self.resource].items()))
        return


class ShowQuota(ShowQuotaBase):
    """Show quotas for a given tenant."""

    def retrieve_data(self, tenant_id, neutron_client):
        return neutron_client.show_quota(tenant_id)


class ShowQuotaDefault(ShowQuotaBase):
    """Show default quotas for a given tenant."""

    def retrieve_data(self, tenant_id, neutron_client):
        return neutron_client.show_quota_default(tenant_id)


class UpdateQuota(neutronV20.NeutronCommand, show.ShowOne):
    """Update a given tenant's quotas."""

    resource = 'quota'

    def get_parser(self, prog_name):
        parser = super(UpdateQuota, self).get_parser(prog_name)
        parser.add_argument(
            '--tenant-id', metavar='tenant-id',
            help=_('The owner tenant ID.'))
        parser.add_argument(
            '--tenant_id',
            help=argparse.SUPPRESS)
        parser.add_argument(
            '--network', metavar='networks',
            help=_('The limit of networks.'))
        parser.add_argument(
            '--subnet', metavar='subnets',
            help=_('The limit of subnets.'))
        parser.add_argument(
            '--port', metavar='ports',
            help=_('The limit of ports.'))
        parser.add_argument(
            '--router', metavar='routers',
            help=_('The limit of routers.'))
        parser.add_argument(
            '--floatingip', metavar='floatingips',
            help=_('The limit of floating IPs.'))
        parser.add_argument(
            '--security-group', metavar='security_groups',
            help=_('The limit of security groups.'))
        parser.add_argument(
            '--security-group-rule', metavar='security_group_rules',
            help=_('The limit of security groups rules.'))
        parser.add_argument(
            '--vip', metavar='vips',
            help=_('The limit of vips.'))
        parser.add_argument(
            '--pool', metavar='pools',
            help=_('The limit of pools.'))
        parser.add_argument(
            '--member', metavar='members',
            help=_('The limit of pool members.'))
        parser.add_argument(
            '--health-monitor', metavar='health_monitors',
            dest='healthmonitor',
            help=_('The limit of health monitors.'))
        parser.add_argument(
            '--loadbalancer', metavar='loadbalancers',
            help=_('The limit of load balancers.'))
        parser.add_argument(
            '--listener', metavar='listeners',
            help=_('The limit of listeners.'))
        parser.add_argument(
            '--rbac-policy', metavar='rbac_policies',
            help=_('The limit of RBAC policies.'))
        parser.add_argument(
            'pos_tenant_id',
            help=argparse.SUPPRESS, nargs='?')

        return parser

    def _validate_int(self, name, value):
        try:
            return_value = int(value)
        except Exception:
            message = (_('Quota limit for %(name)s must be an integer') %
                       {'name': name})
            raise exceptions.CommandError(message=message)
        return return_value

    def args2body(self, parsed_args):
        quota = {}
        for resource in ('network', 'subnet', 'port', 'router', 'floatingip',
                         'security_group', 'security_group_rule',
                         'vip', 'pool', 'member', 'healthmonitor',
                         'loadbalancer', 'listener', 'rbac_policy'):
            if getattr(parsed_args, resource):
                quota[resource] = self._validate_int(
                    resource,
                    getattr(parsed_args, resource))
        if not quota:
            raise exceptions.CommandError(
                message=_('Must specify a valid resource with new quota '
                          'value'))
        return {self.resource: quota}

    def take_action(self, parsed_args):
        neutron_client = self.get_client()
        _extra_values = neutronV20.parse_args_to_dict(self.values_specs)
        neutronV20._merge_args(self, parsed_args, _extra_values,
                               self.values_specs)
        body = self.args2body(parsed_args)
        if self.resource in body:
            body[self.resource].update(_extra_values)
        else:
            body[self.resource] = _extra_values
        obj_updator = getattr(neutron_client,
                              "update_%s" % self.resource)
        tenant_id = get_tenant_id(parsed_args, neutron_client)
        data = obj_updator(tenant_id, body)
        if self.resource in data:
            for k, v in data[self.resource].items():
                if isinstance(v, list):
                    value = ""
                    for _item in v:
                        if value:
                            value += "\n"
                        if isinstance(_item, dict):
                            value += jsonutils.dumps(_item)
                        else:
                            value += str(_item)
                    data[self.resource][k] = value
                elif v is None:
                    data[self.resource][k] = ''
            return zip(*sorted(data[self.resource].items()))
        else:
            return
